@@ -12,6 +12,8 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
+ * 
+ * Copyright Matthias Jentsch 2014-2015
  */
 
 #ifdef HAVE_CONFIG_H
@@ -21,7 +23,6 @@
 #import "TelegramPlugin.h"
 #import "TelegramService.h"
 #import "telegram-purple.h"
-#import <tgl.h>
 
 extern void purple_init_telegram_plugin();
 
@@ -29,9 +30,9 @@ extern void purple_init_telegram_plugin();
 
 - (void) installPlugin
 {
-    pk_path = [self getPkName];
-    purple_init_telegram_plugin();
-    [TelegramService registerService];
+  pk_path = [self getPkName];
+  purple_init_telegram_plugin();
+  [TelegramService registerService];
 }
 
 - (void) installLibpurplePlugin
@@ -63,7 +64,7 @@ extern void purple_init_telegram_plugin();
 
 -(char*)getPkName
 {
-    const char* utf8String = (char *)[[[NSBundle bundleForClass: [self class]] pathForResource: @"tg-server" ofType: @"pub"] UTF8String];
+    const char* utf8String = (char *)[[[NSBundle bundleForClass: [self class]] pathForResource: @"tg-server" ofType: @"tglpub"] UTF8String];
     size_t len = strlen(utf8String) + 1;
     char *buf = malloc(len);
     memcpy(buf, utf8String, len);
