@@ -15,43 +15,93 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 
-    Copyright Matthias Jentsch, Vitaly Valtman, Christopher Althaus, Markus Endres 2014
+    Copyright Matthias Jentsch, Vitaly Valtman, Christopher Althaus, Markus Endres 2014-2015
 */
+
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <fcntl.h>
-#include <stdio.h>
-#include <assert.h>
+#include <cipher.h>
 
-#include <tgl.h>
-#include <tgl-binlog.h>
-
-#include <glib.h>
-#include <request.h>
-#include <openssl/sha.h>
-
-#include "telegram-purple.h"
-#include "msglog.h"
-#include "tgp-2prpl.h"
-#include "tgp-structs.h"
-#include "lodepng/lodepng.h"
+#include "telegram-base.h"
 
+// O_BINARY exists on windows and must be defined, but doesn't exist on unix-based systems
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
 
 #define DC_SERIALIZED_MAGIC 0x868aa81d
 #define STATE_FILE_MAGIC 0x28949a93
 #define SECRET_CHAT_FILE_MAGIC 0x37a1988a
 
+static gboolean read_ui32 (int fd, unsigned int *ret) {
+  typedef char check_int_size[(sizeof (int) >= 4) ? 1 : -1];
+  (void) sizeof (check_int_size);
+
+  unsigned char buf[4];
+  if (4 != read (fd, buf, 4)) {
+    return 0;
+  }
+  // Ugly but works.
+  *ret = 0;
+  *ret |= buf[0];
+  *ret <<= 8;
+  *ret |= buf[1];
+  *ret <<= 8;
+  *ret |= buf[2];
+  *ret <<= 8;
+  *ret |= buf[3];
+  return 1;
+}
+
+int read_pubkey_file (const char *name, struct rsa_pubkey *dst) {
+  // Just to make sure nobody reads garbage.
+  dst->e = 0;
+  dst->n_len = 0;
+  dst->n_raw = NULL;
+
+  int pubkey_fd = open (name, O_RDONLY | O_BINARY);
+  if (pubkey_fd < 0) {
+    return 0;
+  }
+
+  unsigned int e;
+  unsigned int n_len;
+  if (!read_ui32 (pubkey_fd, &e) || !read_ui32 (pubkey_fd, &n_len) // Ensure successful reads
+      || n_len < 128 || n_len > 1024 || e < 5) { // Ensure (at least remotely) sane parameters.
+    close (pubkey_fd);
+    return 0;
+  }
+
+  unsigned char *n_raw = malloc (n_len);
+  if (!n_raw) {
+    close (pubkey_fd);
+    return 0;
+  }
+
+  gint readret;
+  readret = read (pubkey_fd, n_raw, n_len);
+  if (readret <= 0 || (n_len != (guint) readret)) {
+    free (n_raw);
+    close (pubkey_fd);
+    return 0;
+  }
+  close (pubkey_fd);
+
+  dst->e = e;
+  dst->n_len = n_len;
+  dst->n_raw = n_raw;
+  
+  info ("read pubkey file: n_len=%u e=%u", n_len, e);
+  return 1;
+}
 
 void read_state_file (struct tgl_state *TLS) {
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "state") < 0) {
-    return;
-  }
+  name = g_strdup_printf("%s/%s", TLS->base_path, "state");
 
-  int state_file_fd = open (name, O_CREAT | O_RDWR, 0600);
+  int state_file_fd = open (name, O_CREAT | O_RDWR | O_BINARY, 0600);
   free (name);
 
   if (state_file_fd < 0) {
@@ -75,6 +125,7 @@ void read_state_file (struct tgl_state *TLS) {
   bl_do_set_pts (TLS, pts);
   bl_do_set_qts (TLS, qts);
   bl_do_set_date (TLS, date);
+  info ("read state file: seq=%d pts=%d qts=%d date=%d", seq, pts, qts, date);
 }
 
 void write_state_file (struct tgl_state *TLS) {
@@ -85,11 +136,9 @@ void write_state_file (struct tgl_state *TLS) {
   wseq = TLS->seq; wpts = TLS->pts; wqts = TLS->qts; wdate = TLS->date;
   
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "state") < 0) {
-    return;
-  }
+  name = g_strdup_printf("%s/%s", TLS->base_path, "state");
 
-  int state_file_fd = open (name, O_CREAT | O_RDWR, 0600);
+  int state_file_fd = open (name, O_CREAT | O_RDWR | O_BINARY, 0600);
   free (name);
 
   if (state_file_fd < 0) {
@@ -103,23 +152,25 @@ void write_state_file (struct tgl_state *TLS) {
   x[4] = wseq;
   x[5] = wdate;
   assert (write (state_file_fd, x, 24) == 24);
-  close (state_file_fd); 
+  close (state_file_fd);
+  info ("wrote state file: wpts=%d wqts=%d wseq=%d wdate=%d", wpts, wqts, wseq, wdate);
 }
 
-static gboolean write_state_file_gw (gpointer data) {
+static gboolean write_files_gw (gpointer data) {
   struct tgl_state *TLS = data;
   
   ((connection_data *)TLS->ev_base)->write_timer = 0;
-  write_state_file ((struct tgl_state *)TLS);
+  write_state_file (TLS);
+  write_secret_chat_file (TLS);
   
   return FALSE;
 }
 
-void write_state_file_schedule (struct tgl_state *TLS) {
+void write_files_schedule (struct tgl_state *TLS) {
   connection_data *conn = TLS->ev_base;
   
   if (! conn->write_timer) {
-    conn->write_timer = purple_timeout_add (0, write_state_file_gw, TLS);
+    conn->write_timer = purple_timeout_add (0, write_files_gw, TLS);
   }
 }
 
@@ -134,22 +185,20 @@ void write_dc (struct tgl_dc *DC, void *extra) {
     assert (write (auth_file_fd, &x, 4) == 4);
   }
 
-  assert (DC->has_auth);
+  assert (DC->flags & TGLDCF_LOGGED_IN);
 
-  assert (write (auth_file_fd, &DC->port, 4) == 4);
-  int l = strlen (DC->ip);
+  assert (write (auth_file_fd, &DC->options[0]->port, 4) == 4);
+  int l = strlen (DC->options[0]->ip);
   assert (write (auth_file_fd, &l, 4) == 4);
-  assert (write (auth_file_fd, DC->ip, l) == l);
+  assert (write (auth_file_fd, DC->options[0]->ip, l) == l);
   assert (write (auth_file_fd, &DC->auth_key_id, 8) == 8);
   assert (write (auth_file_fd, DC->auth_key, 256) == 256);
 }
 
 void write_auth_file (struct tgl_state *TLS) {
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "auth") < 0) {
-    return;
-  }
-  int auth_file_fd = open (name, O_CREAT | O_RDWR, 0600);
+  name = g_strdup_printf("%s/%s", TLS->base_path, "auth");
+  int auth_file_fd = open (name, O_CREAT | O_RDWR | O_BINARY, 0600);
   free (name);
   if (auth_file_fd < 0) { return; }
   int x = DC_SERIALIZED_MAGIC;
@@ -161,6 +210,7 @@ void write_auth_file (struct tgl_state *TLS) {
 
   assert (write (auth_file_fd, &TLS->our_id, 4) == 4);
   close (auth_file_fd);
+  info ("wrote auth file: magic=%d max_dc_num=%d dc_working_num=%d", x, TLS->max_dc_num, TLS->dc_working_num);
 }
 
 void read_dc (struct tgl_state *TLS, int auth_file_fd, int id, unsigned ver) {
@@ -178,45 +228,43 @@ void read_dc (struct tgl_state *TLS, int auth_file_fd, int id, unsigned ver) {
   assert (read (auth_file_fd, &auth_key_id, 8) == 8);
   assert (read (auth_file_fd, auth_key, 256) == 256);
 
-  bl_do_dc_option (TLS, id, 2, "DC", l, ip, port);
-  bl_do_set_auth_key_id (TLS, id, auth_key);
+  bl_do_dc_option (TLS, 0, id, "DC", 2, ip, l, port);
+  bl_do_set_auth_key (TLS, id, auth_key);
   bl_do_dc_signed (TLS, id);
+  debug ("read dc: id=%d", id);
 }
 
-int error_if_val_false (struct tgl_state *TLS, int val, const char *cause, const char *msg) {
-  if (!val) {
+int tgp_error_if_false (struct tgl_state *TLS, int val, const char *cause, const char *msg) {
+  if (! val) {
     connection_data *conn = TLS->ev_base;
     purple_connection_error_reason (conn->gc, PURPLE_CONNECTION_ERROR_AUTHENTICATION_FAILED, msg);
-    
-    purple_notify_message (_telegram_protocol, PURPLE_NOTIFY_MSG_ERROR,
-            cause, msg, NULL, NULL, NULL);
+    purple_notify_message (_telegram_protocol, PURPLE_NOTIFY_MSG_ERROR, cause, msg, NULL, NULL, NULL);
     return TRUE;
   }
   return 0;
 }
 
 void empty_auth_file (struct tgl_state *TLS) {
+  info ("initializing empty auth file");
   if (TLS->test_mode) {
-    bl_do_dc_option (TLS, 1, 0, "", strlen (TG_SERVER_TEST_1), TG_SERVER_TEST_1, 443);
-    bl_do_dc_option (TLS, 2, 0, "", strlen (TG_SERVER_TEST_2), TG_SERVER_TEST_2, 443);
-    bl_do_dc_option (TLS, 3, 0, "", strlen (TG_SERVER_TEST_3), TG_SERVER_TEST_3, 443);
+    bl_do_dc_option (TLS, 0, 1, "", 0, TG_SERVER_TEST_1, strlen (TG_SERVER_TEST_1), 443);
+    bl_do_dc_option (TLS, 0, 2, "", 0, TG_SERVER_TEST_2, strlen (TG_SERVER_TEST_2), 443);
+    bl_do_dc_option (TLS, 0, 3, "", 0, TG_SERVER_TEST_3, strlen (TG_SERVER_TEST_3), 443);
     bl_do_set_working_dc (TLS, TG_SERVER_TEST_DEFAULT);
   } else {
-    bl_do_dc_option (TLS, 1, 0, "", strlen (TG_SERVER_1), TG_SERVER_1, 443);
-    bl_do_dc_option (TLS, 2, 0, "", strlen (TG_SERVER_2), TG_SERVER_2, 443);
-    bl_do_dc_option (TLS, 3, 0, "", strlen (TG_SERVER_3), TG_SERVER_3, 443);
-    bl_do_dc_option (TLS, 4, 0, "", strlen (TG_SERVER_4), TG_SERVER_4, 443);
-    bl_do_dc_option (TLS, 5, 0, "", strlen (TG_SERVER_5), TG_SERVER_5, 443);
-    bl_do_set_working_dc (TLS, TG_SERVER_DEFAULT);;
+    bl_do_dc_option (TLS, 0, 1, "", 0, TG_SERVER_1, strlen (TG_SERVER_1), 443);
+    bl_do_dc_option (TLS, 0, 2, "", 0, TG_SERVER_2, strlen (TG_SERVER_2), 443);
+    bl_do_dc_option (TLS, 0, 3, "", 0, TG_SERVER_3, strlen (TG_SERVER_3), 443);
+    bl_do_dc_option (TLS, 0, 4, "", 0, TG_SERVER_4, strlen (TG_SERVER_4), 443);
+    bl_do_dc_option (TLS, 0, 5, "", 0, TG_SERVER_5, strlen (TG_SERVER_5), 443);
+    bl_do_set_working_dc (TLS, TG_SERVER_DEFAULT);
   }
 }
 
 void read_auth_file (struct tgl_state *TLS) {
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "auth") < 0) {
-    return;
-  }
-  int auth_file_fd = open (name, O_CREAT | O_RDWR, 0600);
+  name = g_strdup_printf("%s/%s", TLS->base_path, "auth");
+  int auth_file_fd = open (name, O_CREAT | O_RDWR | O_BINARY, 0600);
   free (name);
   if (auth_file_fd < 0) {
     empty_auth_file (TLS);
@@ -250,12 +298,12 @@ void read_auth_file (struct tgl_state *TLS) {
     assert (!l);
   }
   if (our_id) {
-    bl_do_set_our_id (TLS, our_id);
+    bl_do_set_our_id (TLS, TGL_MK_USER (our_id));
   }
   close (auth_file_fd);
+  info ("read auth file: dcs=%d dc_working_num=%d our_id=%d", x, dc_working_num, our_id);
 }
 
-
 void write_secret_chat (tgl_peer_t *_P, void *extra) {
   struct tgl_secret_chat *P = (void *)_P;
   if (tgl_get_peer_type (P->id) != TGL_PEER_ENCR_CHAT) { return; }
@@ -282,14 +330,13 @@ void write_secret_chat (tgl_peer_t *_P, void *extra) {
   assert (write (fd, &P->in_seq_no, 4) == 4);
   assert (write (fd, &P->last_in_seq_no, 4) == 4);
   assert (write (fd, &P->out_seq_no, 4) == 4);
+  debug ("wrote secret chat: %s, state=%d, in_seq_no=%d, out_seq_no=%d", P->print_name, P->state, P->in_seq_no, P->out_seq_no);
 }
 
 void write_secret_chat_file (struct tgl_state *TLS) {
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "secret") < 0) {
-    return;
-  }
-  int secret_chat_fd = open (name, O_CREAT | O_RDWR, 0600);
+  name = g_strdup_printf("%s/%s", TLS->base_path, "secret");
+  int secret_chat_fd = open (name, O_CREAT | O_RDWR | O_BINARY, 0600);
   free (name);
   assert (secret_chat_fd >= 0);
   int x = SECRET_CHAT_FILE_MAGIC;
@@ -307,6 +354,7 @@ void write_secret_chat_file (struct tgl_state *TLS) {
   lseek (secret_chat_fd, 8, SEEK_SET);
   assert (write (secret_chat_fd, &y[1], 4) == 4);
   close (secret_chat_fd);
+  info ("wrote secret chat file: %d chats written.", y[1]);
 }
 
 void read_secret_chat (struct tgl_state *TLS, int fd, int v) {
@@ -317,7 +365,7 @@ void read_secret_chat (struct tgl_state *TLS, int fd, int v) {
   static unsigned char sha[20];
   assert (read (fd, &id, 4) == 4);
   assert (read (fd, &l, 4) == 4);
-  assert (l > 0 && l < 1000);
+  assert (l > 0 && l < 999);
   assert (read (fd, s, l) == l);
   assert (read (fd, &user_id, 4) == 4);
   assert (read (fd, &admin_id, 4) == 4);
@@ -330,6 +378,12 @@ void read_secret_chat (struct tgl_state *TLS, int fd, int v) {
   assert (read (fd, &key, 256) == 256);
   if (v >= 2) {
     assert (read (fd, sha, 20) == 20);
+  } else {
+    PurpleCipher *sha1_cipher = purple_ciphers_find_cipher("sha1");
+    PurpleCipherContext *sha1_ctx = purple_cipher_context_new(sha1_cipher, NULL);
+    purple_cipher_context_append(sha1_ctx, key, 256);
+    purple_cipher_context_digest(sha1_ctx, 20, sha, NULL);
+    purple_cipher_context_destroy(sha1_ctx);
   }
   int in_seq_no = 0, out_seq_no = 0, last_in_seq_no = 0;
   if (v >= 1) {
@@ -338,33 +392,19 @@ void read_secret_chat (struct tgl_state *TLS, int fd, int v) {
     assert (read (fd, &out_seq_no, 4) == 4);
   }
   
-  bl_do_encr_chat_create (TLS, id, user_id, admin_id, s, l);
-  struct tgl_secret_chat  *P = (void *)tgl_peer_get (TLS, TGL_MK_ENCR_CHAT (id));
-  assert (P && (P->flags & FLAG_CREATED));
-  bl_do_encr_chat_set_date (TLS, P, date);
-  bl_do_encr_chat_set_ttl (TLS, P, ttl);
-  bl_do_encr_chat_set_layer (TLS ,P, layer);
-  bl_do_encr_chat_set_state (TLS, P, state);
-  bl_do_encr_chat_set_key (TLS, P, key, key_fingerprint);
-  if (v >= 2) {
-    bl_do_encr_chat_set_sha (TLS, P, sha);
-  } else {
-    SHA1 ((void *)key, 256, sha);
-    bl_do_encr_chat_set_sha (TLS, P, sha);
-  }
-  if (v >= 1) {
-    bl_do_encr_chat_set_seq (TLS, P, in_seq_no, last_in_seq_no, out_seq_no);
-  }
-  bl_do_encr_chat_set_access_hash (TLS, P, access_hash);
+  s[l] = '\0';
+  debug ("read secret chat: %s, state=%d, in_seq_no=%d, last_in_seq_no=%d, out_seq_no=%d",
+      s, state, in_seq_no, last_in_seq_no, out_seq_no);
+  bl_do_encr_chat (TLS, id, &access_hash, &date, &admin_id, &user_id, key, NULL, sha, &state, &ttl,
+      &layer, &in_seq_no, &last_in_seq_no, &out_seq_no, &key_fingerprint, TGLECF_CREATE | TGLECF_CREATED,
+      s, l);
 }
 
 void read_secret_chat_file (struct tgl_state *TLS) {
   char *name = 0;
-  if (asprintf (&name, "%s/%s", TLS->base_path, "secret") < 0) {
-    return;
-  }
+  name = g_strdup_printf("%s/%s", TLS->base_path, "secret");
   
-  int secret_chat_fd = open (name, O_RDWR, 0600);
+  int secret_chat_fd = open (name, O_RDWR | O_BINARY, 0600);
   free (name);
   
   if (secret_chat_fd < 0) { return; }
@@ -377,195 +417,95 @@ void read_secret_chat_file (struct tgl_state *TLS) {
   assert (v == 0 || v == 1 || v == 2); // version
   assert (read (secret_chat_fd, &x, 4) == 4);
   assert (x >= 0);
+  int cnt = x;
   while (x -- > 0) {
     read_secret_chat (TLS, secret_chat_fd, v);
   }
   close (secret_chat_fd);
+  info ("read secret chat file: %d chats read", cnt);
 }
 
-void telegram_export_authorization (struct tgl_state *TLS);
-void export_auth_callback (struct tgl_state *TLS, void *extra, int success) {
-  if (!error_if_val_false(TLS, success, "Login Canceled", "Authentication export failed.")) {
-    telegram_export_authorization (TLS);
+gchar *get_config_dir (char const *username) {
+  gchar *dir = g_strconcat (purple_user_dir(), G_DIR_SEPARATOR_S, config_dir,
+                                G_DIR_SEPARATOR_S, username, NULL);
+  
+  if (g_str_has_prefix (dir, g_get_tmp_dir())) {
+    // telepathy-haze will set purple user dir to a tmp path,
+    // but we need the files to be persistent
+    g_free (dir);
+    dir = g_strconcat (g_get_home_dir(), G_DIR_SEPARATOR_S, ".telegram-purple",
+                                  G_DIR_SEPARATOR_S, username, NULL);
   }
+  g_mkdir_with_parents (dir, 0700);
+  return dir;
 }
 
-void telegram_export_authorization (struct tgl_state *TLS) {
-  int i;
-  for (i = 0; i <= TLS->max_dc_num; i++) if (TLS->DC_list[i] && !tgl_signed_dc (TLS, TLS->DC_list[i])) {
-    debug ("tgl_do_export_auth(%d)", i);
-    tgl_do_export_auth (TLS, i, export_auth_callback, (void*)(long)TLS->DC_list[i]);    
+gchar *get_user_pk_path () {
+  /*
+     This can't be conditional on whether or not we're using telepathy, because
+     then we would need to make sure that `make local_install` also knows about
+     that location. So we *always* use ${HOME}/.purple/telegram-purple,
+     even when the other files aren't in this folder.
+     Note that this is only visible when using Telepathy/Empathy with
+     local_install, which should be kinda rare anyway (use telepathy-morse!).
+   */
+  return g_strconcat (g_get_home_dir(), G_DIR_SEPARATOR_S, ".purple",
+                                G_DIR_SEPARATOR_S, "telegram-purple",
+                                G_DIR_SEPARATOR_S, user_pk_filename, NULL);
+}
+
+gchar *get_download_dir (struct tgl_state *TLS) {
+  assert (TLS->base_path);
+  static gchar *dir;
+  if (dir) {
+    g_free (dir);
+  }
+  dir = g_strconcat (TLS->base_path, G_DIR_SEPARATOR_S, "downloads", NULL);
+  g_mkdir_with_parents (dir, 0700);
+  return dir;
+}
+
+void write_secret_chat_gw (struct tgl_state *TLS, void *extra, int success, struct tgl_secret_chat *_) {
+  if (!success) {
+    tgp_notify_on_error_gw (TLS, NULL, success);
     return;
   }
-  write_auth_file (TLS);
-  on_ready (TLS);
+  write_secret_chat_file (TLS);
 }
 
-static void request_code (struct tgl_state *TLS);
-static void request_name_and_code (struct tgl_state *TLS);
-static void code_receive_result (struct tgl_state *TLS, void *extra, int success, struct tgl_user *U) {
-  if (!success) {
-    debug ("Bad code...");
-    request_code (TLS);
-  } else {
-    telegram_export_authorization (TLS);
-  }
-}
-
-static void code_auth_receive_result (struct tgl_state *TLS, void *extra, int success, struct tgl_user *U) {
-  if (!success) {
-    debug ("Bad code...");
-    request_name_and_code (TLS);
-  } else {
-    telegram_export_authorization (TLS);
-  }
-}
-
-void request_code_entered (gpointer data, const gchar *code) {
-  struct tgl_state *TLS = data;
-  connection_data *conn = TLS->ev_base;
-  char const *username = purple_account_get_username(conn->pa);
-  tgl_do_send_code_result (TLS, username, conn->hash, code, code_receive_result, 0) ;
-}
-
-static void request_code_canceled (gpointer data) {
-  struct tgl_state *TLS = data;
-  connection_data *conn = TLS->ev_base;
-
-  purple_connection_error_reason(conn->gc,
-      PURPLE_CONNECTION_ERROR_AUTHENTICATION_FAILED, "registration canceled");
-}
-
-static void request_name_code_entered (PurpleConnection* gc, PurpleRequestFields* fields) {
-  connection_data *conn = purple_connection_get_protocol_data(gc);
-  struct tgl_state *TLS = conn->TLS;
-  char const *username = purple_account_get_username(conn->pa);
-  
-  const char* first = purple_request_fields_get_string(fields, "first_name");
-  const char* last = purple_request_fields_get_string(fields, "last_name");
-  const char* code = purple_request_fields_get_string(fields, "code");
-  if (!first || !last || !code) {
-    request_name_and_code (TLS);
-    return;
-  }
-  
-  tgl_do_send_code_result_auth (TLS, username, conn->hash, code, first, last, code_auth_receive_result, NULL);
-}
-
-static void request_code (struct tgl_state *TLS) {
-  debug ("Client is not registered, registering...");
-  connection_data *conn = TLS->ev_base;
-  int compat = purple_account_get_bool (tg_get_acc(TLS), "compat-verification", 0);
-  
-  if (compat || ! purple_request_input (conn->gc, "Telegram Code", "Enter Telegram Code",
-        "Telegram wants to verify your identity, please enter the code, that you have received via SMS.",
-        NULL, 0, 0, "code", "OK", G_CALLBACK(request_code_entered), "Cancel",
-        G_CALLBACK(request_code_canceled), conn->pa, NULL, NULL, TLS)) {
-
-    // purple request API is not available, so we create a new conversation (the Telegram system
-    // account "7770000") to prompt the user for the code
-          
-    conn->in_fallback_chat = 1;
-    purple_connection_set_state (conn->gc, PURPLE_CONNECTED);
-    PurpleConversation *conv = purple_conversation_new (PURPLE_CONV_TYPE_IM, conn->pa, "777000");
-    purple_conversation_write (conv, "777000", "What is your SMS verification code?",
-          PURPLE_MESSAGE_RECV | PURPLE_MESSAGE_SYSTEM, 0);
-  }
-}
-
-static void request_name_and_code (struct tgl_state *TLS) {
-  debug ("Phone is not registered, registering...");
-
-  connection_data *conn = TLS->ev_base;
-
-  PurpleRequestFields* fields = purple_request_fields_new();
-  PurpleRequestField* field = 0;
-
-  PurpleRequestFieldGroup* group = purple_request_field_group_new("Registration");
-  field = purple_request_field_string_new("first_name", "First Name", "", 0);
-  purple_request_field_group_add_field(group, field);
-  field = purple_request_field_string_new("last_name", "Last Name", "", 0);
-  purple_request_field_group_add_field(group, field);
-  purple_request_fields_add_group(fields, group);
-
-  group = purple_request_field_group_new("Authorization");
-  field = purple_request_field_string_new("code", "Telegram Code", "", 0);
-  purple_request_field_group_add_field(group, field);
-  purple_request_fields_add_group(fields, group);
-
-  if (!purple_request_fields (conn->gc, "Register", "Please register your phone number.", NULL, fields, "Ok",
-    G_CALLBACK( request_name_code_entered ), "Cancel", NULL, conn->pa, NULL, NULL, conn->gc)) {
-    // purple_request API not available
-    const char *error = "Phone number is not registered, please register your phone on a different client.";
-    purple_connection_error_reason (conn->gc, PURPLE_CONNECTION_ERROR_AUTHENTICATION_FAILED, error);
-    purple_notify_error(_telegram_protocol, "Not Registered", "Not Registered", error);
-  }
-}
-
-static void sign_in_callback (struct tgl_state *TLS, void *extra, int success, int registered, const char *mhash) {
-  connection_data *conn = TLS->ev_base;
-  if (!error_if_val_false (TLS, success, "Invalid phone number",
-          "Please enter only numbers in the international phone number format, "
-          "a leading + following by the country prefix and the phone number.\n"
-          "Do not use any other special chars.")) {
-    conn->hash = strdup (mhash);
-    if (registered) {
-      request_code (TLS);
+void tgp_create_group_chat_by_usernames (struct tgl_state *TLS, const char *title, const char **users, int num_users,
+                                         int use_print_names) {
+  tgl_peer_id_t ids[num_users + 1];
+  int i, j = 0;
+  ids[j++] = TLS->our_id;
+  for (i = 0; i < num_users; i++) if (str_not_empty (users[i])) {
+    tgl_peer_t *P = NULL;
+    if (use_print_names) {
+       // used by Adium autocompletion that is based on print_names
+      P = tgl_peer_get_by_name (TLS, users[i]);
     } else {
-      request_name_and_code (TLS);
+      P = tgp_blist_lookup_peer_get (TLS, users[i]);
+    }
+    if (P && tgl_get_peer_id (P->id) != tgl_get_peer_id (TLS->our_id)) {
+      debug ("Adding %s: %d", P->print_name, tgl_get_peer_id (P->id));
+      ids[j++] = P->id;
+    } else {
+      debug ("User %s not found in peer list", users[j]);
     }
   }
-}
-
-static void telegram_send_sms (struct tgl_state *TLS) {
-  if (tgl_signed_dc (TLS, TLS->DC_working)) {
-    telegram_export_authorization (TLS);
-    return;
-  }
-  connection_data *conn = TLS->ev_base;
-  char const *username = purple_account_get_username(conn->pa);
-  tgl_do_send_code (TLS, username, sign_in_callback, 0);
-}
-
-static int all_authorized (struct tgl_state *TLS) {
-  int i;
-  for (i = 0; i <= TLS->max_dc_num; i++) if (TLS->DC_list[i]) {
-    if (!tgl_authorized_dc (TLS, TLS->DC_list[i])) {
-      return 0;
-    }
-  }
-  return 1;
-}
-
-static int check_all_authorized (gpointer arg) {
-  struct tgl_state *TLS = arg;
-
-  if (all_authorized (TLS)) {
-    ((connection_data *)TLS->ev_base)->login_timer = 0;
-    telegram_send_sms (TLS);    
-    return FALSE;
+  if (j > 1) {
+    tgl_do_create_group_chat (TLS, j, ids, title, (int) strlen(title),
+                              tgp_notify_on_error_gw, g_strdup (title));
   } else {
-    return TRUE;
+    purple_notify_message (_telegram_protocol, PURPLE_NOTIFY_MSG_INFO, _("Couldn't create group"), _("Please select at least one other user."), NULL,
+                           NULL, NULL);
   }
 }
-    
-void telegram_login (struct tgl_state *TLS) {
-  connection_data *conn = TLS->ev_base;
-  
-  read_auth_file (TLS);
-  read_state_file (TLS);
-  read_secret_chat_file (TLS);
-  if (all_authorized (TLS)) {
-    telegram_send_sms (TLS);
-    return;
-  }
-  conn->login_timer = purple_timeout_add (100, check_all_authorized, TLS);
-}
 
 /**
  * This function generates a png image to visualize the sha1 key from an encrypted chat.
  */
-int generate_ident_icon (struct tgl_state *TLS, unsigned char* sha1_key) {
+int tgp_visualize_key (struct tgl_state *TLS, unsigned char* sha1_key) {
   int colors[4] = {
     0xffffff,
     0xd5e6f3,
@@ -585,7 +525,7 @@ int generate_ident_icon (struct tgl_state *TLS, unsigned char* sha1_key) {
       int offset = bitpointer / 8;
       int shiftOffset = bitpointer % 8;
       int val = sha1_key[offset + 3] << 24 | sha1_key[offset + 2] << 16 | sha1_key[offset + 1] << 8 | sha1_key[offset];
-      idx = abs((val >> shiftOffset) & 3) % 4;
+      idx = abs ((val >> shiftOffset) & 3) % 4;
       bitpointer += 2;
       unsigned offset_x = x * 4 * (img_size / 8);
       for (i = 0; i < img_size / 8; i++)
@@ -594,23 +534,27 @@ int generate_ident_icon (struct tgl_state *TLS, unsigned char* sha1_key) {
         for (j = 0; j < img_size / 8; j++)
         {
           unsigned off_x = offset_x + j * 4;
-          image[off_y + off_x + 0] = (colors[idx] >> 16) & 0xFF;
+          image[off_y + off_x + 0] = colors[idx] & 0xFF;
           image[off_y + off_x + 1] = (colors[idx] >> 8) & 0xFF;
-          image[off_y + off_x + 2] = colors[idx] & 0xFF;
+          image[off_y + off_x + 2] = (colors[idx] >> 16) & 0xFF;
           image[off_y + off_x + 3] = 0xFF;
         }
       }
     }
   }
-  unsigned char* png;
-  size_t pngsize;
-  unsigned error = lodepng_encode32(&png, &pngsize, image, img_size, img_size);
-  int imgStoreId = -1;
-  if(!error)
-  {
-    imgStoreId = purple_imgstore_add_with_id (png, pngsize, NULL);
-    used_images_add ((connection_data*)TLS->ev_base, imgStoreId);
-  }
-  g_free(image);
+  int imgStoreId = p2tgl_imgstore_add_with_id_raw(image, img_size, img_size);
+  used_images_add ((connection_data*)TLS->ev_base, imgStoreId);
+  g_free (image);
   return imgStoreId;
 }
+
+void tgp_notify_on_error_gw (struct tgl_state *TLS, void *extra, int success) {
+  if (!success) {
+    char *errormsg = g_strdup_printf ("%d: %s", TLS->error_code, TLS->error);
+    failure (errormsg);
+    purple_notify_message (_telegram_protocol, PURPLE_NOTIFY_MSG_ERROR, _("Query Failed"),
+                           errormsg, NULL, NULL, NULL);
+    g_free (errormsg);
+    return;
+  }
+}
